%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dynamic Syntax
%
% flags
%
% Matthew Purver, 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


:- dynamic flag/2.


default_flags :-
	% do we print all trees in forest, or only the complete ones? (all/some/complete,none)
	default_flag( print, some ),
	% do we print in HTML format, or plain text? (txt/html)
	default_flag( format, txt ),
	% do we print to the terminal or to network sockets? (term/sock)
	default_flag( io, term ),
	% how many sentences of context do we maintain? (all/one/none)
	default_flag( context, one ),
	% how do we search during generation? (d=depth-first/b=breadth-first)
	default_flag( search, d ),
	% how many generation alternatives do we try for? (all/one)
	default_flag( gen, one ),
	% do we use lexical pre-selection? (true/false)
	default_flag( lexbag, false ),
	% allowed VP ellipsis types (strict/all)
	default_flag( ellip, strict ).


setflag( Flag, Value ) :-
	retractall( flag( Flag, _ ) ),
	assert( flag( Flag, Value ) ).


default_flag( Flag, Value ) :-
	(
	  flag( Flag, _ )
	->
	  true
	;
	  setflag( Flag, Value )
	).


% process_args/0
% Gets command-line arguments or sets defaults for display format etc.
process_args :-
	prolog_version( sicstus ),
	current_prolog_flag( argv, ArgList ),
	process_args( ArgList ).
process_args :-
	prolog_version( swi ),
	current_prolog_flag( argv, [_Command | ArgList] ),
	process_args( ArgList ).

% process_args/1 - process_args( +ArgList )
% Asserts any suitable command-line arguments
process_args( [] ).
process_args( [H | T] ) :-
	(
	  prolog_version( sicstus )
	->
	  name( H, Chars ),
	  read_from_chars( Chars, Term )
	;
	  atom_to_term( H, Term, _ )
	),
	(
	  callable( Term )
	->
	  call( Term )
	;
	  true
	),
	process_args( T ).
