%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dynamic Syntax
%
% generator
%
% Matthew Purver, 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% some predefined trees for testing
gen_trees( [
	    ( snore(Man), tree([node('0',[+pres,ty(t),fo(snore(Man))]),node('01',[ty(e>t),fo(A^snore(A))]),node('00',[+(male),ty(e),fo(Man)])],'0') ),
	    ( like(Man,Woman), tree([node('0',[+pres,ty(t),fo(like(Man,Woman))]),node('01',[ty(e>t),fo(A^like(A,Woman))]),node('010',[+(female),ty(e),fo(Woman)]),node('011',[ty(e>(e>t)),fo(B^C^like(C,B))]),node('00',[+(male),ty(e),fo(Man)])],'0') )
	   ] ).

% top-level: pull out the complete trees and generate them with gen/5
generate( GoalForest, Strings, Method ) :-
	complete_trees( GoalForest, GoalTrees ),
	root_forest( RootForest ),
	gen( GoalTrees, RootForest, Method, Strings, _Forest ).

generate( GoalForest, Strings, Method, Forest ) :-
	complete_trees( GoalForest, GoalTrees ),
	root_forest( RootForest ),
	gen( GoalTrees, RootForest, Method, Strings, Forest ).

generate( GoalForest, StartForest, Strings, Method, Forest ) :-
	complete_trees( GoalForest, GoalTrees ),
%	root_forest( RootForest ),
%	append( StartForest, RootForest, InputForest ),
	gen( GoalTrees, StartForest, Method, Strings, Forest ).


% base case
gen( [], _Forest1, _Method, [], _Forest2 ).
% depth-first, one solution
gen( [H | _T], Forest1, [d, one], [String], Forest2 ) :-
	lexical_selection( H, W ),
	d_gen( H, W, Forest1, Forest2, String ).
% depth-first, all solutions
gen( [H | _T], Forest1, [d, all], Strings, _Forest2 ) :-
	lexical_selection( H, W ),
	findall( String, d_gen( H, W, Forest1, _Forest2, String ), Strings ).
% breadth-first
gen( [H | _T], Root, [b, Number], Strings, _Forest2 ) :-
	lexical_selection( H, W ),
	b_gen( H, W, [Root], Strings, Number ).


% main generation predicate. Simple or what?
main_gen( GoalTree, Word, Actions, Forest1, Forest2 ) :-
	findall( twa( Tree2, Words2, Actions2 ), (
						   member( twa( Tree1, Words1, Actions1 ), Forest1 ),
						   apply( Actions, Tree1, Tree2 ),
						   subsumes( GoalTree, Tree2 ),
						   append( Words1, [Word], Words2 ),
						   append( Actions1, Actions, Actions2 )
						 ), Forest2 ),
	Forest2 \= [],
	Forest2 = [twa(_,Words2,_) | _],
	print( Words2 ), newline.


% d_gen/2 - d_gen( +LogicalForm, ?String )
% just for testing
d_gen( Sem, String ) :-
	gen_trees( GT ),
	member( ( Sem, GoalTree ), GT ),
	root_forest( Forest ),
	d_gen( GoalTree, Forest, String ).

% d_gen/6 - gen( +GoalTree, ?WordBag, +Forest1, ?Forest2, ?FinalString )
% depth-first search
d_gen( _GoalTree, _WordBag, Forest, Forest, [] ) :-
	complete_forest( Forest, [twa( _Tree, Words, _Actions ) | _] ),
	newline, print( Words ), newline.
d_gen( GoalTree, WordBag, Forest1, Forest3, [Word | Words] ) :-
	choose_word( Word, Actions, WordBag ),
	main_gen( GoalTree, Word, Actions, Forest1, Forest2 ),
	d_gen( GoalTree, WordBag, Forest2, Forest3, Words ).


% b_gen/2 - b_gen( +LogicalForm, ?StringList )
% just for testing
b_gen( Sem, StringList, Number ) :-
	gen_trees( GT ),
	member( ( Sem, GoalTree ), GT ),
	root_forest( Forest ),
	b_gen( GoalTree, Forest, StringList, Number ).


% b_gen/5 - breadth-first version with lexical pre-selection
b_gen( _GoalTree, _WordBag, [], [], _Number ).
b_gen( GoalTree, WordBag, Forests1, Strings, Number ) :-
	findall( Forest2, (
			    member( Forest1, Forests1 ),
			    member( Word, WordBag ),
			    word( Word, Actions ),
			    main_gen( GoalTree, Word, Actions, Forest1, Forest2 )
			  ), Forests2 ),
	findall( String, (
			   member( Forest, Forests2 ),
			   complete_forest( Forest, [twa( _Tree, String, _Actions ) | _] ),
			   newline, print( String ), newline
			 ), Strings2 ),
	(
	  Number = one,
	  Strings2 \= []
	->
	  Strings = Strings2
	;
	  b_gen( GoalTree, WordBag, Forests2, Strings3, Number ),
	  append( Strings2, Strings3, Strings )
	).


% if NOT doing lexical pre-selection (i.e. strictly incremental)
choose_word( Word, Actions, WordBag ) :-
	var( WordBag ),
	(
	  word_in_context( Word, Actions );
	  word( Word, Actions )
	).
% if doing lexical preselection
choose_word( Word, Actions, WordBag ) :-
	nonvar( WordBag ),
	(
	  member( Word, WordBag ),
	  word_in_context( Word, Actions );
	  member( Word, WordBag ),
	  word( Word, Actions )
	).


% choose a multiset of words
% NOT currently enabled by default - although it speeds up generation, it is not strictly incremental
% (can't handle online goal tree modification) and doesn't fit well with alignment model
lexical_selection( tree( Nodes, _Pointer ), WordBag ) :-
	(
	  flag( lexbag, true )
	->
	  lexical_selection( Nodes, WordBag1 ),
	  remove_duplicates( WordBag1, WordBag ),
	  print( 'Bag: ' ), print( WordBag ), newline
	;
	  true
	).

% hack to pre-select
% wh-relatives could be chosen properly on basis of LINK structure in goal tree
% pronouns, ellipsis auxs could be chosen on the fly given labels
lexical_selection( [], [] ).
%lexical_selection( [], [to] ).
%lexical_selection( [], [who, which, that, he, she, him, her, does] ).

lexical_selection( [node( _N, Labels ) | NTail], WordBag ) :-
	findall( Word, (
			 member( fo( Formula ), Labels ),
			 Formula \= e(_),
			 word( Word, Action ),
			 associated( Action, fo( Formula ) ),
			 \+ (
			      member( +Label, Labels ),
			      \+ associated( Action, +Label )
			    )
		       ), Words ),
	lexical_selection( NTail, WTail ),
	append( Words, WTail, WordBag ).


associated( put( Label1 ), Label2 ) :-
	variant( Label1, Label2 ).

associated( ( if _List1, then List2, else List3 ), Label ) :-
	associated( List2, Label );
	associated( List3, Label ).

associated( [Head | Tail], Label ) :-
	associated( Head, Label );
	associated( Tail, Label ).
