%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Dynamic Syntax
%
% parser
%
% Matthew Purver, 2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% parse/3 - parse( +Forest1, +WordList, ?Forest2 )
% Adds words in WordList one by one, proceeding from the input forest Forest1
% to the resulting output forest Forest2
parse( Forest, [], Forest ).
parse( Forest1, [H | T], Forest3 ) :-
	parse_word( Forest1, H, Forest2 ),
	parse( Forest2, T, Forest3 ).

% parse_word( +Forest1, +Word, -Forest2 )
% Adds a single word, extending the input forest Forest1 to the resulting
% possible output forest Forest2
parse_word( _Forest1, Word, [] ) :-
	atom( Word ),
	\+ word( Word, _Action ),
	print( 'unknown word: ' ), print( Word ), newline.
parse_word( Forest1, Word, Forest2 ) :-
	atom( Word ),
	print( 'word: ' ), print( Word ), newline,
	findall( twa( Tree2, Words2, Actions2 ), (
						   member( twa( Tree1, Words1, Actions1 ), Forest1 ),
						   word( Word, Actions ),
						   apply( Actions, Tree1, Tree2 ),
						   append( Words1, [Word], Words2 ),
						   append( Actions1, Actions, Actions2 )
						 ), Forest ),
	remove_variants( Forest, Forest2 ).

% apply/3 - apply( +Actions, +Tree1, ?Tree2 )
% Applies list of Actions to Tree1, resulting in Tree2

% recurse through a list of actions, allowing computational actions
apply( [], Tree, Tree ).
apply( [H | T], Tree1, Tree4 ) :-
	copy_term( H, A ), % preventing unification of stored actions with metavariable values
	exec( A, Tree1, Tree2 ),
	adjust( Tree2, Tree3 ),
	apply( T, Tree3, Tree4 ).

% remove_variants/2
% remove_variants(+List, ?Pruned)
% Pruned is the result of removing all *variant* (identical modulo variables) elements in List.
remove_variants( [], [] ).
remove_variants( [H | T1], T2 ) :-
	member( X, T1 ),
	variant( X, H ),
	!,
	remove_variants( T1, T2 ).
remove_variants( [H | T1], [H | T2] ) :-
	remove_variants( T1, T2 ).
